; ***  S U R R O U N D  ***
; Copyright 1977? Atari
; Designer: Alan Miller

; Analyzed, labeled and commented
;  by Thomas Jentzsch (JTZ)
; Last Update: 25.08.2001 (v0.1)

    PROCESSOR 6502
    INCLUDE vcs.h


;===============================================================================
; A S S E M B L E R - S W I T C H E S
;===============================================================================

FILL_OPT        = 1             ; fill optimized bytes with NOPs (unused)
NTSC            = 1             ; compile for NTSC


;===============================================================================
; C O N S T A N T S
;===============================================================================

NUM_MODES       = 14
NUM_PLAYERS     = 2

NUM_ROWS        = 20            ; number of playfield rows (blocks)
ROW_H           = 9             ; height of playfield row (block)
DIGIT_H         = 5             ; size of digit data
SCORE_H         = 6             ; y-size of score
PF_W            = 40            ; width of playfield

; gameMode flags:
NOGRAFFITI      = %10000000
SPEEDUP         = %01000000
NOGRAFFITI2     = %00100000     ; always same as NOGRAFFITI
SINGLEPLAYER    = %00010000
WRAPAROUND      = %00001000
ERASE           = %00000100
DIAGONAL        = %00000001

; mask for SWCHB:
BW_MASK         = %1000         ; black and white bit
SELECT_MASK     = %0010         ; select bit

; joystick bits:
MOVE_RIGHT      = %10000000
MOVE_LEFT       = %01000000
MOVE_DOWN       = %00100000
MOVE_UP         = %00010000


;===============================================================================
; Z P - V A R I A B L E S
;===============================================================================

; 100 bytes that store the playfield:
PF0Lst          = $80           ; ..$93         used for left and right part
PF1LstL         = $94           ; ..$a7
PF2LstL         = $a8           ; ..$bb
PF1LstR         = $bc           ; ..$cf
PF2LstR         = $d0           ; ..$e3

; many of the following variables are used for different temporary data:
;               = $e4           ;               flags?
blockEnd        = $e5
PFLstPtr        = blockEnd      ; ..$e6

temp1           = $e7
temp2           = $e8

gameSpeed       = $ec
yPos0           = $ed           ;               x-position of player 0
yPos1           = $ee           ;               x-position of player 1
xPos0           = $ef           ;               y-position of player 0
xPos1           = $f0           ;               y-position of player 1
gameMode        = $f1           ;               flag for game variations
oldMove0        = $f2           ;               saved old joystick directions of player 0?
oldMove1        = $f3           ;               saved old joystick directions of player 1?

score0          = $f6           ;               BCD score of player 0
score1          = $f7           ;               BCD score of player 1

scoreMask       = $f8           ;               hides right socre
gameNumber      = $f9           ;               0..13
;               = $fb           ;               some more flags?
oldDirs         = $fc           ;               joystick again!?


;===============================================================================
; M A C R O S
;===============================================================================

  MAC FILL_NOP
    IF FILL_OPT
      REPEAT {1}
         NOP
      REPEND
    ENDIF
  ENDM


;===============================================================================
; R O M - C O D E (Part 1)
;===============================================================================

    ORG $f000

DrawKernel SUBROUTINE
.waitTim:
    lda    INTIM            ; 4
    bne    .waitTim         ; 2+1
    sta    WSYNC            ; 3
    sta    VBLANK           ; 3
    lda    #$02             ; 2                 enable score mode
    sta    CTRLPF           ; 3

; wait three extra lines:
    ldx    #$03             ; 2
.waitLines:
    sta    WSYNC            ; 3
    sta    HMCLR            ; 3
    dex                     ; 2
    bne    .waitLines       ; 2

; code for displaying the score (3 lines kernel!):
    ldx    #SCORE_H-1       ; 2
    lda    #$00             ; 2
    sta    temp1            ; 3
    sta    temp2            ; 3
.loopScore:
    sta    WSYNC            ; 3
;---------------------------------------
    lda    temp1            ; 3
    sta    PF1              ; 3                 player 0 score at left
    lda    score0           ; 3
    and    #$f0             ; 2                 superfluous! (LSR involved again :)
    lsr                     ; 2
    lsr                     ; 2
    lsr                     ; 2
    lsr                     ; 2
    tay                     ; 2
    lda    DigitOfsTab,y    ; 4
    clc                     ; 2
    adc    DigitOfs2Tab,x   ; 4
    tay                     ; 2
    lda    DigitTab,y       ; 4
    and    #$f0             ; 2
    sta    temp1            ; 3
    lda    temp2            ; 3
    sta    PF1              ; 3                 player 1 score at right
    lda    score0           ; 3
    and    #$0f             ; 2
    tay                     ; 2
    lda    DigitOfsTab,y    ; 4
    clc                     ; 2
    adc    DigitOfs2Tab,x   ; 4
    sta    WSYNC            ; 3
;---------------------------------------
    tay                     ; 2
    lda    DigitTab,y       ; 4
    and    #$0f             ; 2
    ora    temp1            ; 3
    sta    temp1            ; 3
    sta    PF1              ; 3                 player 0 score at left
    lda    score1           ; 3
    and    #$f0             ; 2
    lsr                     ; 2
    lsr                     ; 2
    lsr                     ; 2
    lsr                     ; 2
    tay                     ; 2
    lda    DigitOfsTab,y    ; 4
    ldy    temp2            ; 3
    sty    PF1              ; 3                 player 1 score at right
    clc                     ; 2
    adc    DigitOfs2Tab,x   ; 4
    tay                     ; 2
    lda    DigitTab,y       ; 4
    and    #$f0             ; 2
    sta    temp2            ; 3
    lda    score1           ; 3
    and    #$0f             ; 2
    sta    WSYNC            ; 3
;---------------------------------------
    tay                     ; 2
    lda    DigitOfsTab,y    ; 4
    ldy    temp1            ; 3
    sty    PF1              ; 3                 player 0 score at left
    clc                     ; 2
    adc    DigitOfs2Tab,x   ; 4
    tay                     ; 2
    lda    DigitTab,y       ; 4
    and    #$0f             ; 2
    and    scoreMask        ; 3                 mask right score while selecting game
    and    scoreMask        ; 3                 waste 3 cycles!
    ora    temp2            ; 3
    sta    temp2            ; 3
    sta    PF1              ; 3                 player 1 score at right
    dex                     ; 2
    bmi    .exitScore       ; 2
    jmp    .loopScore       ; 3

.exitScore:
    sta    WSYNC            ; 3
;---------------------------------------
    lda    #$00             ; 2                 start with empty playfield
    sta    PF0              ; 3
    sta    PF1              ; 3
    sta    PF2              ; 3
    sta    CTRLPF           ; 3                 non reflected playfield

; y ist used as line counter inside the main kernel:
    ldy    #56              ; 2                 count from 56 up to 256
    lda    #58              ; 2
    sta    blockEnd         ; 3
    ldx    #-1              ; 2
.loopPF:
    cpy    #59              ; 2                 draw three empty lines at the top
    bcc    .skipKernel      ; 2+1
    cpy    blockEnd         ; 3                 end of current row?
    bcs    .endRow          ; 2                 yes, end row
.loopPFRow:
    sta    WSYNC            ; 3
;---------------------------------------
    lda    PF0Lst,x         ; 4                 use lower bits for left PF0 pattern
    asl                     ; 2
    asl                     ; 2
    asl                     ; 2
    asl                     ; 2
    sta    PF0              ; 3
    lda    PF1LstL,x        ; 4
    sta    PF1              ; 3
    lda    PF2LstL,x        ; 4
    sta    PF2              ; 3
    lda    PF0Lst,x         ; 4                 use upper bits for right PF0 pattern
    sta    PF0              ; 3
    lda    PF1LstR,x        ; 4
    sta    PF1              ; 3
    iny                     ; 2                 increase line counter
    lda    PF2LstR,x        ; 4
    sta    PF2              ; 3
    cpy    blockEnd         ; 3                 end of current row?
    bcc    .loopPFRow       ; 2                 no, continue loop
.endRow:
    inx                     ; 2
    lda    #%1111           ; 2                 pattern for player 0 cursor
    cpx    yPos0            ; 3                 y-position of player 0 cursor?
    beq    .drawGRP0        ; 2                 yes, draw cursor
    lda    #$00             ; 2                  no, draw nothing
.drawGRP0:
    sta    WSYNC            ; 3
    sta    GRP0             ; 3
    lda    #%1111           ; 2                 pattern for player 1 cursor
    cpx    yPos1            ; 3                 y-position of player 0 cursor?
    beq    .drawGRP1        ; 2                 yes, draw cursor
    lda    #$00             ; 2                  no, draw nothing
.drawGRP1:
    sta    GRP1             ; 3
    lda    #$00             ; 2                 draw one empty playfield line between rows
    sta    PF0              ; 3
    sta    PF1              ; 3
    sta    PF2              ; 3
    tya                     ; 2                 calculate next blockEnd
    clc                     ; 2
    adc    #ROW_H           ; 2
    sta    blockEnd         ; 3
.contKernel:
    iny                     ; 2                 end of kernel?
    beq    .exitKernel      ; 2                 yes, exit
    cpy    #232             ; 2                 end of bottom border?
    bcc    .loopPF          ; 2+1               no, loop
.skipKernel:
    lda    #$ff             ; 2                  yes, draw one filled line
    sta    WSYNC            ; 3
    sta    PF0              ; 3
    sta    PF1              ; 3
    sta    PF2              ; 3
    lda    #$00             ; 2                 disable player graphics after each row
    sta    GRP0             ; 3
    sta    GRP1             ; 3
    beq    .contKernel      ; 3

.exitKernel:
; superfluos code (missiles aren't used!):
    lda    #$00             ; 2
    sta    ENAM0            ; 3
    sta    ENAM1            ; 3
    rts                     ; 6

VerticalSync SUBROUTINE
    lda    #42              ; 2
    sta    HMCLR            ; 3
    sta    WSYNC            ; 3
    sta    VBLANK           ; 3                 start vertical blank
    sta    VSYNC            ; 3                 start vertical sync
    sta    TIM8T            ; 4
    inc    $e4              ; 5
    inc    $fd              ; 5
; here Alan is wasting a LOT of not needed CPU time!:
.waitTim:
    lda    INTIM            ; 4
    bne    .waitTim         ; 2
    sta    WSYNC            ; 3
    sta    VSYNC            ; 3                 end vertical sync
    lda    #40              ; 2
    sta    TIM64T           ; 4
    rts                     ; 6

SinglePlayer SUBROUTINE
; AI routines?
    lda    #$ff             ; 2
    sta    $e9              ; 3
    sta    $eb              ; 3
    lda    #$06             ; 2
    bit    SWCHB            ; 4                 player 0 difficulty?
    bvs    .pro0            ; 2                 pro, skip!
    lda    #$03             ; 2
.pro0:
    sta    $e6              ; 3
    lda    #$00             ; 2
    sta    temp2            ; 3
    ldy    xPos0            ; 3
    lda    PFLstTab,y       ; 4
    sta    temp1            ; 3
    ldx    #$10             ; 2
    stx    $ea              ; 3
    ldx    xPos0            ; 3
    ldy    yPos0            ; 3
Lf166:
    inc    $eb              ; 5
    dey                     ; 2
    bmi    Lf172            ; 2
    lda    PFPatTab,x       ; 4
    and    (temp1),y        ; 5
    beq    Lf166            ; 2
Lf172:
    lda    $e6              ; 3
    cmp    $eb              ; 3
    bcs    Lf17a            ; 2
    sta    $eb              ; 3
Lf17a:
    ldy    yPos0            ; 3
    ldx    xPos0            ; 3
Lf17e:
    inc    $e9              ; 5
    iny                     ; 2
    cpy    #$15             ; 2
    bcs    Lf18c            ; 2
    lda    PFPatTab,x       ; 4
    and    (temp1),y        ; 5
    beq    Lf17e            ; 2
Lf18c:
    lda    $e9              ; 3
    cmp    $eb              ; 3
    bcc    Lf1a8            ; 2
    ldx    $eb              ; 3
    cpx    $e6              ; 3
    bcc    Lf1a2            ; 2
    bit    $fd              ; 3
    bvs    Lf1a8            ; 2
    cmp    $e6              ; 3
    bcc    Lf1a2            ; 2
    lda    $e6              ; 3
Lf1a2:
    sta    $eb              ; 3
    ldx    #$20             ; 2
    stx    $ea              ; 3
Lf1a8:
    ldy    yPos0            ; 3
    ldx    xPos0            ; 3
    lda    #$ff             ; 2
    sta    $e9              ; 3
Lf1b0:
    inc    $e9              ; 5
    dex                     ; 2
    bmi    Lf1c1            ; 2
    lda    PFLstTab,x       ; 4
    sta    temp1            ; 3
    lda    (temp1),y        ; 5
    and    PFPatTab,x       ; 4
    beq    Lf1b0            ; 2
Lf1c1:
    lda    $e9              ; 3
    cmp    $eb              ; 3
    bcc    Lf1dd            ; 2
    ldx    $eb              ; 3
    cpx    $e6              ; 3
    bcc    Lf1d7            ; 2
    bit    $fd              ; 3
    bvs    Lf1dd            ; 2
    cmp    $e6              ; 3
    bcc    Lf1d7            ; 2
    lda    $e6              ; 3
Lf1d7:
    sta    $eb              ; 3
    ldx    #$40             ; 2
    stx    $ea              ; 3
Lf1dd:
    ldy    yPos0            ; 3
    ldx    xPos0            ; 3
    lda    #$ff             ; 2
    sta    $e9              ; 3
Lf1e5:
    inc    $e9              ; 5
    inx                     ; 2
    cpx    #$29             ; 2
    beq    Lf1f8            ; 2
    lda    PFLstTab,x       ; 4
    sta    temp1            ; 3
    lda    (temp1),y        ; 5
    and    PFPatTab,x       ; 4
    beq    Lf1e5            ; 2
Lf1f8:
    lda    $e9              ; 3
    cmp    $eb              ; 3
    bcc    Lf212            ; 2+1
    ldx    $eb              ; 3
    cpx    $e6              ; 3
    bcc    Lf20e            ; 2
    bit    $fd              ; 3
    bvs    Lf212            ; 2
    cmp    $e6              ; 3
    bcc    Lf20e            ; 2
    lda    $e6              ; 3
Lf20e:
    lda    #$80             ; 2
    sta    $ea              ; 3
Lf212:
    lda    $ea              ; 3
    eor    #$ff             ; 2
    and    #$f0             ; 2
    sta    $ea              ; 3
    lda    oldDirs          ; 3
    and    #$0f             ; 2
    ora    $ea              ; 3
    sta    oldDirs          ; 3
    jmp    ContSingle       ; 3

CheckJoysticks SUBROUTINE
    lda    gameMode         ; 3
    and    #SINGLEPLAYER    ; 2
    cmp    #SINGLEPLAYER    ; 2                 superfluos!
    bne    TwoPlayers       ; 2
    jmp    SinglePlayer     ; 3

TwoPlayers SUBROUTINE
    lda    gameMode         ; 3                 NOGRAFFITI?
    bpl    .grafittiMode    ; 2
ContSingle:
    lda    oldDirs          ; 3
    eor    #$ff             ; 2
    sta    temp2            ; 3
    and    #$f0             ; 2                 mask for player 0
    sta    $e6              ; 3
; mix previous joystick directions of both players into temp1:
    lda    oldMove1         ; 3
    lsr                     ; 2                 hey, no superfluous code here :)
    lsr                     ; 2
    lsr                     ; 2
    lsr                     ; 2
    ora    oldMove0         ; 3
    sta    temp1            ; 3

    ldx    #NUM_PLAYERS     ; 2

; check difficulty of player 0:
    bit    SWCHB            ; 4                 difficulty player 0?
    bvs    .pro0            ; 2                 pro, skip!

; this code prevents player 0 from backing into previous track block:
    and    #$f0             ; 2                  amateur, isolate bits for player 0
    cmp    #MOVE_UP         ; 2
    beq    .contAmateur     ; 2
    cmp    #MOVE_DOWN       ; 2
    beq    .contAmateur     ; 2
    cmp    #MOVE_LEFT       ; 2
    beq    .contAmateur     ; 2
    cmp    #MOVE_RIGHT      ; 2
    beq    .contAmateur     ; 2
.pro0:
    dex                     ; 2
.loopPlayers:
; check difficulty of player 1:
    bit    SWCHB            ; 4                 difficulty player 1?
    bmi    .pro1            ; 2                 pro, skip!

; this code prevents player 1 from backing into previous track block:
    lda    temp2            ; 3
    and    #$0f             ; 2                 mask for player 1
    sta    $e6              ; 3
    lda    temp1            ; 3
    and    #$0f             ; 2
    cmp    #MOVE_UP>>4      ; 2
    beq    .contAmateur     ; 2
    cmp    #MOVE_DOWN>>4    ; 2
    beq    .contAmateur     ; 2
    cmp    #MOVE_LEFT>>4    ; 2
    beq    .contAmateur     ; 2
    cmp    #MOVE_RIGHT>>4   ; 2
    bne    .pro1            ; 2
.contAmateur:
    lda    temp1            ; 3
    asl                     ; 2
    and    $e6              ; 3
    and    #%10101010       ; 2
    eor    temp2            ; 3
    sta    temp2            ; 3
    lda    temp1            ; 3
    lsr                     ; 2
    and    $e6              ; 3
    and    #%01010101       ; 2
    eor    temp2            ; 3
    sta    temp2            ; 3
.pro1:
    lda    temp2            ; 3
    dex                     ; 2
    bne    .loopPlayers     ; 2
    eor    #$ff             ; 2
    jmp    Lf2a4            ; 3

.grafittiMode:
; here starts a (to?) complicated joystick routine:
    lda    SWCHA            ; 4
Lf2a4:
    ldx    #0               ; 2                 x = player index
    sta    $e6              ; 3                 save joystick value
.loopPlayer:
    ldy    #0               ; 2
    sty    $e5              ; 3                 clear movement flag
    asl                     ; 2
    bcs    .skipRight       ; 2
    lda    #MOVE_RIGHT      ; 2
    sta    oldMove0,x       ; 4
    jmp    .contRight       ; 3

.skipRight:
    asl                     ; 2
    bcs    .skipLeft        ; 2
    lda    #MOVE_LEFT       ; 2
    sta    oldMove0,x       ; 4
    jmp    .contLeft        ; 3

.skipLeft:
    asl                     ; 2
    bcs    .skipDown        ; 2
    lda    #MOVE_DOWN       ; 2
    sta    oldMove0,x       ; 4
    jmp    .contDown        ; 3

.skipDown:
    asl                     ; 2
    bcs    .skipUp          ; 2
    lda    #MOVE_UP         ; 2
    sta    oldMove0,x       ; 4
    jmp    .contUp          ; 3

.skipUp:
    lda    gameMode         ; 3
    asl                     ; 2                 NOGRAFFITI?
    bcs    .useOldDir       ; 2                 yes, continue movement
    jmp    .contLoop        ; 3

.useOldDir:
    lda    oldMove0,x       ; 4
    asl                     ; 2
    bcc    .skipOldRight    ; 2
    inc    $e5              ; 5
    jmp    .contRight       ; 3

.skipOldRight:
    asl                     ; 2
    bcc    .skipOldLeft     ; 2
    inc    $e5              ; 5
    jmp    .contLeft        ; 3

.skipOldLeft:
    asl                     ; 2
    bcc    .skipOldDown     ; 2
    jmp    .contDown        ; 3

.skipOldDown:
    asl                     ; 2
    bcc    .skipOldUp       ; 2
    jmp    .contUp          ; 3

.skipOldUp:
    jmp    .contLoop        ; 3

.contDown:
    lda    yPos0,x          ; 4
    clc                     ; 2
    adc    #1               ; 2
    sbc    #NUM_ROWS-1      ; 2
    bmi    noBottom         ; 2
    lda    #0               ; 2
    sta    yPos0,x          ; 4
    beq    .contLoop        ; 3

noBottom:
    adc    #NUM_ROWS        ; 2
    sta    yPos0,x          ; 4
    jmp    .contLoop        ; 3

.contUp:
    lda    yPos0,x          ; 4
    sec                     ; 2
    sbc    #1               ; 2
    bpl    noTop            ; 2
    lda    #NUM_ROWS-1      ; 2
noTop:
    sta    yPos0,x          ; 4
    jmp    .contLoop        ; 3

Lf321:
    lda    gameMode         ; 3
    ror                     ; 2                 DIAGONAL?
    bcc    .contLoop        ; 2
    lda    #1               ; 2
    cmp    $e5              ; 3                 this should always be truew
    bne    Lf335            ; 2
    ldy    #$00             ; 2
    lda    oldMove0,x       ; 4
    asl                     ; 2
    asl                     ; 2
    jmp    .skipOldLeft     ; 3

Lf335:
    lda    $e6              ; 3                 restore joystick value
    cpx    #1               ; 2
    bcc    .notPlayer1      ; 2
    asl                     ; 2
    asl                     ; 2
    asl                     ; 2
    asl                     ; 2
.notPlayer1:
    asl                     ; 2
    asl                     ; 2
    asl                     ; 2
    bcs    Lf34f            ; 2
    lda    #MOVE_DOWN       ; 2
    ora    oldMove0,x       ; 4
    and    #~MOVE_UP        ; 2
    sta    oldMove0,x       ; 4
    jmp    .contDown        ; 3

Lf34f:
    asl                     ; 2
    bcs    .contLoop        ; 2
    lda    #MOVE_UP         ; 2
    ora    oldMove0,x       ; 4
    and    #~MOVE_DOWN      ; 2
    sta    oldMove0,x       ; 4
    jmp    .contUp          ; 3

.contLoop:
    inx                     ; 2
    cpx    #NUM_PLAYERS     ; 2
    bcs    .exitLoop        ; 2
    lda    $e6              ; 3
    asl                     ; 2
    asl                     ; 2
    asl                     ; 2
    asl                     ; 2
    jmp    .loopPlayer      ; 3

.contRight:
    lda    #$c0             ; 2
    sta    HMP0,x           ; 4
    inc    xPos0,x          ; 6
    lda    #PF_W            ; 2
    cmp    xPos0,x          ; 4
    bne    .noBorder        ; 2
    lda    #0               ; 2
    sta    xPos0,x          ; 4
    jmp    Lf321            ; 3

.contLeft:
    lda    #$40             ; 2
    sta    HMP0,x           ; 4
    dec    xPos0,x          ; 6
    bpl    .noBorder        ; 2
    lda    #PF_W-1          ; 2
    sta    xPos0,x          ; 4
.noBorder:
    jmp    Lf321            ; 3

.exitLoop:
    lda    #$f0             ; 2
    ora    gameSpeed        ; 3
    sta    gameSpeed        ; 3
    rts                     ; 6

START SUBROUTINE
    sei                     ; 2
    cld                     ; 2
    lda    #$10             ; 2
    sta    scoreMask        ; 3                 used later to recognize start
    jmp    InitRound        ; 3

.mainLoop:
    jsr    VerticalSync     ; 6
    lda    scoreMask        ; 3
    bmi    Lf3a7            ; 2
    jmp    Lf431            ; 3

Lf3a7:
    lda    SWCHA            ; 4
    and    #$f0             ; 2
    cmp    #$f0             ; 2
    beq    Lf3ba            ; 2
    sta    $e5              ; 3
    lda    oldDirs          ; 3
    and    #$0f             ; 2
    ora    $e5              ; 3
    sta    oldDirs          ; 3
Lf3ba:
    lda    SWCHA            ; 4
    and    #$0f             ; 2
    cmp    #$0f             ; 2
    beq    Lf3cd            ; 2
    sta    $e5              ; 3
    lda    oldDirs          ; 3
    and    #$f0             ; 2
    ora    $e5              ; 3
    sta    oldDirs          ; 3
Lf3cd:
    lda    gameMode         ; 3
    and    #NOGRAFFITI2     ; 2
    beq    .skipCollisions  ; 2
    jsr    CheckCollisions  ; 6
.skipCollisions:
    bit    gameMode         ; 3                 SPEEDUP?
    bvc    .skipSpeedUp     ; 2                 no, skip
    lda    $e4              ; 3
    bne    .skipSpeedUp     ; 2
    lda    gameSpeed        ; 3
    and    #$0f             ; 2
    cmp    #$04             ; 2
    bcc    .skipSpeedUp     ; 2
    dec    gameSpeed        ; 5
    dec    gameSpeed        ; 5
    dec    gameSpeed        ; 5
.skipSpeedUp:
    lda    gameSpeed        ; 3
    and    #$f0             ; 2
    cmp    #$20             ; 2
    bne    Lf3fe            ; 2
    bit    $fb              ; 3
    bmi    Lf3fe            ; 2
    bvs    Lf3fe            ; 2
    lda    #$00             ; 2
    sta    AUDV0            ; 3
Lf3fe:
    lda    gameSpeed        ; 3
    and    #$f0             ; 2
    cmp    #$00             ; 2
    bne    Lf431            ; 2
    jsr    UpdatePlayfield  ; 6
    jsr    CheckJoysticks   ; 6
    sta    WSYNC            ; 3
    sta    HMOVE            ; 3
    lda    gameSpeed        ; 3
    and    #$0f             ; 2
    sta    gameSpeed        ; 3
    asl                     ; 2
    asl                     ; 2
    asl                     ; 2
    asl                     ; 2
    ora    gameSpeed        ; 3
    sta    gameSpeed        ; 3

; player sound when player move:
    lda    #$08             ; 2
    bit    gameMode         ; 3                 NOGRAFFITI?
    bmi    .skipGraffiti    ; 2
    lda    #$00             ; 2
.skipGraffiti:
    sta    AUDV0            ; 3
    lda    #$0c             ; 2
    sta    AUDC0            ; 3
    lda    gameSpeed        ; 3
    asl                     ; 2
    sta    AUDF0            ; 3
Lf431:
    jsr    Lf542            ; 6
Lf434:
    jsr    DrawKernel       ; 6
    lda    gameSpeed        ; 3
    sec                     ; 2
    sbc    #$10             ; 2
    sta    gameSpeed        ; 3
    jmp    .mainLoop        ; 3

UpdatePlayfield SUBROUTINE
; updates the playfield variables
    ldx    #0               ; 2
    stx    PFLstPtr+1       ; 3
.loopUpdate:
    ldy    xPos0,x          ; 4
    lda    PFLstTab,y       ; 4
    sta    PFLstPtr         ; 3
    lda    PFPatTab,y       ; 4
    sta    temp1            ; 3
    ldy    yPos0,x          ; 4
    lda    gameMode         ; 3                 ERASE?
    lsr                     ; 2
    lsr                     ; 2
    lsr                     ; 2
    bcc    .skipErase       ; 2                 yes, set new block
    lda    INPT4,x          ; 4                 fire pressed?
    bmi    .skipErase       ; 2                 no, set new block
    lda    temp1            ; 3                  yes, invert block
    eor    #$ff             ; 2
    and    (PFLstPtr),y     ; 5
    sta    (PFLstPtr),y     ; 6
    jmp    .contLoop        ; 3

.skipErase:
    lda    temp1            ; 3
    ora    (PFLstPtr),y     ; 5
    sta    (PFLstPtr),y     ; 6
.contLoop:
    inx                     ; 2
    cpx    #NUM_PLAYERS     ; 2                 both players complete?
    bcc    .loopUpdate      ; 2                 no, loop again
    rts                     ; 6

InitRound SUBROUTINE
    ldx    #$ff             ; 2
    lda    #$00             ; 2
    sta    PF0              ; 3
    sta    PF1              ; 3
    sta    PF2              ; 3
    txs                     ; 2
    jsr    DrawKernel       ; 6
    jsr    VerticalSync     ; 6

; clear playfield:
    lda    #$00             ; 2
    ldx    #NUM_ROWS*4      ; 2
.loopClear:
    sta    PF1LstL-1,x      ; 4
    dex                     ; 2
    bne    .loopClear       ; 2
    lda    gameMode         ; 3
    and    #WRAPAROUND      ; 2
    bne    .wrapAround      ; 2

; set left and right columns of playfield:
    ldx    #NUM_ROWS        ; 2
    lda    #%00000001       ; 2
.loopLeft:
    sta    PF0Lst-1,x       ; 4
    dex                     ; 2
    bne    .loopLeft        ; 2

    ldx    #NUM_ROWS        ; 2
    lda    #%10000000       ; 2
.loopRight:
    sta    PF2LstR-1,x      ; 4
    dex                     ; 2
    bne    .loopRight       ; 2

; set top and bottom row of playfield:
    lda    #$ff             ; 2
    sta    PF0Lst           ; 3
    sta    PF0Lst+NUM_ROWS-1; 3
    sta    PF1LstL          ; 3
    sta    PF1LstL+NUM_ROWS-1;3
    sta    PF2LstL          ; 3
    sta    PF2LstL+NUM_ROWS-1;3
    sta    PF1LstR          ; 3
    sta    PF1LstR+NUM_ROWS-1;3
    sta    PF2LstR          ; 3
    sta    PF2LstR+NUM_ROWS-1;3
    jmp    .contInit        ; 3

.wrapAround:
; clear borders:
    lda    #$00             ; 2
    ldx    #NUM_ROWS        ; 2
.loopClearBorders:
    sta    PF0Lst-1,x       ; 4
    sta    PF2LstR-1,x      ; 4
    dex                     ; 2
    bne    .loopClearBorders; 2

.contInit:
    lda    #$00             ; 2
    ldx    #42              ; 2
.loopClearTIA:
    sta    $43,x            ; 4                 ???
    dex                     ; 2
    bne    .loopClearTIA    ; 2

; position players:
    lda    #10              ; 2
    sta    yPos0            ; 3
    sta    yPos1            ; 3
    sta    xPos0            ; 3
    lda    #30              ; 2
    sta    xPos1            ; 3
    sta    $fb              ; 3
    sta    WSYNC            ; 3
    ldy    #5               ; 2
.waitLines:
    dey                     ; 2
    bpl    .waitLines       ; 2
    sta    RESP0            ; 3
    sta    WSYNC            ; 3
    ldy    #$0a             ; 2
.waitPos:
    dey                     ; 2
    bpl    .waitPos         ; 2
    sta    RESP1            ; 3
    lda    #$30             ; 2
    sta    HMP0             ; 3
    lda    #$e0             ; 2
    sta    HMP1             ; 3
    sta    WSYNC            ; 3
    sta    HMOVE            ; 3
    lda    #$ff             ; 2
    sta    oldDirs          ; 3
    sta    gameSpeed        ; 3
    sta    AUDF0            ; 3
    lda    gameNumber       ; 3
    and    #$07             ; 2
    asl                     ; 2
    asl                     ; 2
    tay                     ; 2
    ldx    #$00             ; 2
    lda    SWCHB            ; 4
    and    #BW_MASK         ; 2
    bne    .colorMode       ; 2
    ldy    #$20             ; 2
.colorMode:
.loopColors:
    lda    ColorTab,y       ; 4
    sta    COLUP0,x         ; 4
    iny                     ; 2
    inx                     ; 2
    cpx    #$04             ; 2
    bcc    .loopColors      ; 2
    lda    #$00             ; 2
    sta    AUDV0            ; 3
    sta    AUDV1            ; 3
    sta    $e4              ; 3
    lda    gameMode         ; 3
    and    #SINGLEPLAYER    ; 2
    bne    .singlePlayer    ; 2
    lda    #MOVE_RIGHT      ; 2
    sta    oldMove0         ; 3
.singlePlayer:
    lda    #MOVE_LEFT       ; 2
    sta    oldMove1         ; 3
    lda    #$0c             ; 2
    sta    AUDC0            ; 3
    jmp    Lf434            ; 3

Lf542 SUBROUTINE
    lda    scoreMask        ; 3
    cmp    #$10             ; 2                 game just staret?
    bne    .noStart         ; 2                 no, skip
    lda    #$ff             ; 2                  yes, reset..
    sta    gameNumber       ; 3                  ..gamenumber
    bne    .contStart       ; 3

.noStart:
    lda    SWCHB            ; 4
    ror                     ; 2                 another Alan Miller LSR problem? ;-)
    bcs    .skipReset       ; 2
    lda    #$ff             ; 2
    sta    scoreMask        ; 3
; clear scores:
    lda    #$00             ; 2
    sta    score0           ; 3
    sta    score1           ; 3
    sta    $f5              ; 3
    sta    $f4              ; 3
    lda    $e4              ; 3
    and    #$01             ; 2
    sta    $e4              ; 3
    lda    #$0f             ; 2
    sta    $fa              ; 3
    jmp    InitRound        ; 3

.skipReset:
    lda    $e4              ; 3
    and    #$3f             ; 2
    bne    Lf577            ; 2
    sta    $f5              ; 3
Lf577:
    lda    $e4              ; 3
    and    #$ff             ; 2                 ???
    bne    Lf585            ; 2
    inc    $f4              ; 5
    bne    Lf585            ; 2
    lda    #$00             ; 2
    sta    scoreMask        ; 3
Lf585:
    lda    SWCHB            ; 4
    and    #SELECT_MASK     ; 2
    beq    .doSelect        ; 2
    sta    $f5              ; 3
    bne    Lf5d0            ; 3

.doSelect:
    bit    $f5              ; 3
    bmi    Lf5d0            ; 2
    lda    #$ff             ; 2
    sta    $f5              ; 3
    inc    gameNumber       ; 5
.contStart:
; convert game number into BCD number:
    ldx    #$00             ; 2
    stx    score0           ; 3
    ldx    gameNumber       ; 3
.loopModes:
    sed                     ; 2
    lda    score0           ; 3
    clc                     ; 2
    adc    #$01             ; 2
    sta    score0           ; 3
    cld                     ; 2
    dex                     ; 2
    bne    .loopModes       ; 2

    ldx    #$00             ; 2
    stx    score1           ; 3
    stx    $fa              ; 3
    stx    scoreMask        ; 3
    stx    $e4              ; 3
    lda    gameNumber       ; 3
    cmp    #NUM_MODES       ; 2
    bcc    .skipWrap        ; 2
    stx    score0           ; 3
    stx    gameNumber       ; 3
.skipWrap:
    sed                     ; 2
    clc                     ; 2
    lda    score0           ; 3
    adc    #$01             ; 2
    sta    score0           ; 3
    cld                     ; 2
    ldx    gameNumber       ; 3
    lda    GameModeTab,x    ; 4
    sta    gameMode         ; 3
Lf5d0:
    lda    gameNumber       ; 3
    and    #$07             ; 2
    asl                     ; 2
    asl                     ; 2
    tay                     ; 2
    ldx    #$00             ; 2
    lda    scoreMask        ; 3
    and    #$f0             ; 2
    eor    #$ff             ; 2
    and    $f4              ; 3
    sta    temp2            ; 3
    lda    #$ff             ; 2
    sta    temp1            ; 3
    lda    SWCHB            ; 4
    and    #BW_MASK         ; 2
    bne    .colorMode       ; 2
    lda    #$0f             ; 2
    sta    temp1            ; 3
    ldy    #$20             ; 2
.colorMode:
    lda    ColorTab,y       ; 4
    eor    temp2            ; 3
    and    temp1            ; 3
    bit    scoreMask        ; 3
    bvs    Lf605            ; 2+1
    sta    COLUP0,x         ; 4
    lda    #$00             ; 2
    sta    AUDV0            ; 3
Lf605:
    iny                     ; 2
    inx                     ; 2
    cpx    #$04             ; 2
    bcc    .colorMode       ; 2+1
    lda    #$00             ; 2
    sta    PF0              ; 3
    sta    PF1              ; 3
    sta    PF2              ; 3
    rts                     ; 6

CheckCollisions SUBROUTINE
; check collison of player 0:
    lda    CXP0FB           ; 3
    bpl    .notWon1         ; 2
    bit    $fb              ; 3
    bvs    Lf65a            ; 2
    lda    #$7f             ; 2
    sta    $fb              ; 3
; increase score of player 1:
    sed                     ; 2
    lda    score1           ; 3
    clc                     ; 2
    adc    #$01             ; 2
    cld                     ; 2
    sta    score1           ; 3
    cmp    #$10             ; 2
    bne    .notWon1         ; 2
; player 1 won:
    lda    #$0f             ; 2
    sta    scoreMask        ; 3
.notWon1:
; check collison of player 1:
    lda    CXP1FB           ; 3
    bpl    .notWon0         ; 2
    bit    $fb              ; 3
    bmi    Lf65a            ; 2
    lda    $fb              ; 3
    ora    #$80             ; 2
    sta    $fb              ; 3
; increase score of player 0:
    sed                     ; 2
    lda    score0           ; 3
    clc                     ; 2
    adc    #$01             ; 2
    cld                     ; 2
    sta    score0           ; 3
    cmp    #$10             ; 2
    bne    .notWon0         ; 2
; player 0 won:
    lda    #$0f             ; 2
    sta    scoreMask        ; 3
.notWon0:
    lda    $fb              ; 3
    and    #$c0             ; 2
    beq    .noCollision     ; 2
    jmp    Lf6e1            ; 3

.noCollision:
    rts                     ; 6

; ** sound routines: ***
Lf65a:
    lda    $e4              ; 3
    and    #$03             ; 2
    bne    Lf66a            ; 2
    lda    #$01             ; 2
    sta    AUDC0            ; 3
    lda    #$06             ; 2
    sta    AUDF0            ; 3
    bne    Lf672            ; 3

Lf66a:
    lda    $fb              ; 3
    sta    AUDV0            ; 3
    lda    #$0e             ; 2
    sta    AUDF0            ; 3
Lf672:
    lda    $e4              ; 3
    and    #$0f             ; 2
    bne    Lf6e1            ; 2
    dec    $fb              ; 5
    lda    $fb              ; 3
    and    #$0f             ; 2
    bne    Lf683            ; 2
    jmp    InitRound        ; 3

Lf683:
    lda    gameNumber       ; 3
    and    #$07             ; 2
    asl                     ; 2
    asl                     ; 2
    tay                     ; 2
    lda    SWCHB            ; 4
    and    #BW_MASK         ; 2
    bne    .colorMode       ; 2
    ldy    #$20             ; 2
.colorMode:
    bit    $fb              ; 3
    bpl    Lf6bc            ; 2
    lda    $fb              ; 3
    and    #$20             ; 2
    bne    Lf6ad            ; 2
    lda    $fb              ; 3
    ora    #$20             ; 2
    sta    $fb              ; 3
    iny                     ; 2
    lda    ColorTab,y       ; 4
    dey                     ; 2
    sta    COLUP1           ; 3
    jmp    Lf6bc            ; 3

Lf6ad:
    lda    $fb              ; 3
    eor    #$20             ; 2
    sta    $fb              ; 3
    iny                     ; 2
    iny                     ; 2
    lda    ColorTab,y       ; 4
    dey                     ; 2
    dey                     ; 2
    sta    COLUP1           ; 3
Lf6bc:
    bit    $fb              ; 3
    bvc    Lf6e1            ; 2
    lda    $fb              ; 3
    and    #$10             ; 2
    bne    Lf6d4            ; 2
    lda    $fb              ; 3
    ora    #$10             ; 2
    sta    $fb              ; 3
    lda    ColorTab,y       ; 4
    sta    COLUP0           ; 3
    jmp    Lf6e1            ; 3

Lf6d4:
    lda    $fb              ; 3
    eor    #$10             ; 2
    sta    $fb              ; 3
    iny                     ; 2
    iny                     ; 2
    lda    ColorTab,y       ; 4
    sta    COLUP0           ; 3
Lf6e1:
    lda    gameSpeed        ; 3
    clc                     ; 2
    adc    #$10             ; 2
    sta    gameSpeed        ; 3
    rts                     ; 6


;===============================================================================
; R O M - T A B L E S
;===============================================================================

PFLstTab:
; converts x-position into playfield variables:
    .byte PF0Lst, PF0Lst, PF0Lst, PF0Lst
    .byte PF1LstL, PF1LstL, PF1LstL, PF1LstL, PF1LstL, PF1LstL, PF1LstL, PF1LstL
    .byte PF2LstL, PF2LstL, PF2LstL, PF2LstL, PF2LstL, PF2LstL, PF2LstL, PF2LstL
    .byte PF0Lst, PF0Lst, PF0Lst, PF0Lst
    .byte PF1LstR, PF1LstR, PF1LstR, PF1LstR, PF1LstR, PF1LstR, PF1LstR, PF1LstR
    .byte PF2LstR, PF2LstR, PF2LstR, PF2LstR, PF2LstR, PF2LstR, PF2LstR, PF2LstR

PFPatTab:
; pattern for left PF0:
    .byte %00000001 ; |       X|
    .byte %00000010 ; |      X |
    .byte %00000100 ; |     X  |
    .byte %00001000 ; |    X   |
; pattern for left PF1:
    .byte %10000000 ; |X       |
    .byte %01000000 ; | X      |
    .byte %00100000 ; |  X     |
    .byte %00010000 ; |   X    |
    .byte %00001000 ; |    X   |
    .byte %00000100 ; |     X  |
    .byte %00000010 ; |      X |
    .byte %00000001 ; |       X|
; pattern for left PF2:
    .byte %00000001 ; |       X|
    .byte %00000010 ; |      X |
    .byte %00000100 ; |     X  |
    .byte %00001000 ; |    X   |
    .byte %00010000 ; |   X    |
    .byte %00100000 ; |  X     |
    .byte %01000000 ; | X      |
    .byte %10000000 ; |X       |
; pattern for right PF0:
    .byte %00010000 ; |   X    |
    .byte %00100000 ; |  X     |
    .byte %01000000 ; | X      |
    .byte %10000000 ; |X       |
; pattern for right PF1:
    .byte %10000000 ; |X       |
    .byte %01000000 ; | X      |
    .byte %00100000 ; |  X     |
    .byte %00010000 ; |   X    |
    .byte %00001000 ; |    X   |
    .byte %00000100 ; |     X  |
    .byte %00000010 ; |      X |
    .byte %00000001 ; |       X|
; pattern for right PF2:
    .byte %00000001 ; |       X|
    .byte %00000010 ; |      X |
    .byte %00000100 ; |     X  |
    .byte %00001000 ; |    X   |
    .byte %00010000 ; |   X    |
    .byte %00100000 ; |  X     |
    .byte %01000000 ; | X      |
    .byte %10000000 ; |X       |

DigitTab:
Zero:
    .byte %00001110 ; |    XXX |
    .byte %00001010 ; |    X X |
    .byte %00001010 ; |    X X |
    .byte %00001010 ; |    X X |
    .byte %00001110 ; |    XXX |
One:
    .byte %00100010 ; |  X   X |
    .byte %00100010 ; |  X   X |
    .byte %00100010 ; |  X   X |
    .byte %00100010 ; |  X   X |
    .byte %00100010 ; |  X   X |
Two:
    .byte %11101110 ; |XXX XXX |
    .byte %00100010 ; |  X   X |
    .byte %11101110 ; |XXX XXX |
    .byte %10001000 ; |X   X   |
    .byte %11101110 ; |XXX XXX |
Three:
    .byte %11101110 ; |XXX XXX |
    .byte %00100010 ; |  X   X |
    .byte %01100110 ; | XX  XX |
    .byte %00100010 ; |  X   X |
    .byte %11101110 ; |XXX XXX |
Four:
    .byte %10101010 ; |X X X X |
    .byte %10101010 ; |X X X X |
    .byte %11101110 ; |XXX XXX |
    .byte %00100010 ; |  X   X |
    .byte %00100010 ; |  X   X |
Five:
    .byte %11101110 ; |XXX XXX |
    .byte %10001000 ; |X   X   |
    .byte %11101110 ; |XXX XXX |
    .byte %00100010 ; |  X   X |
    .byte %11101110 ; |XXX XXX |
Six:
    .byte %11101110 ; |XXX XXX |
    .byte %10001000 ; |X   X   |
    .byte %11101110 ; |XXX XXX |
    .byte %10101010 ; |X X X X |
    .byte %11101110 ; |XXX XXX |
Seven:
    .byte %11101110 ; |XXX XXX |
    .byte %00100010 ; |  X   X |
    .byte %00100010 ; |  X   X |
    .byte %00100010 ; |  X   X |
    .byte %00100010 ; |  X   X |
Eight:
    .byte %11101110 ; |XXX XXX |
    .byte %10101010 ; |X X X X |
    .byte %11101110 ; |XXX XXX |
    .byte %10101010 ; |X X X X |
    .byte %11101110 ; |XXX XXX |
Nine:
    .byte %11101110 ; |XXX XXX |
    .byte %10101010 ; |X X X X |
    .byte %11101110 ; |XXX XXX |
    .byte %00100010 ; |  X   X |
    .byte %11101110 ; |XXX XXX |

ColorTab:
    .byte $c8, $84, $3a, $44
    .byte $46, $ea, $5a, $88
    .byte $2a, $46, $b4, $e8
    .byte $3a, $86, $e8, $46
    .byte $2a, $56, $e8, $a4
    .byte $36, $2c, $c8, $a2
    .byte $c8, $2a, $83, $46
    .byte $86, $2c, $48, $e8
    .byte $00, $0e, $04, $08    ; b/w "colors"

DigitOfsTab:
    .byte DIGIT_H*0, DIGIT_H*1, DIGIT_H*2, DIGIT_H*3, DIGIT_H*4
    .byte DIGIT_H*5, DIGIT_H*6, DIGIT_H*7, DIGIT_H*8, DIGIT_H*9

; used to duplicate bottom row of score:
DigitOfs2Tab:
    .byte 4, 4, 3, 2, 1, 0

GameModeTab:
    .byte NOGRAFFITI|        NOGRAFFITI2
    .byte NOGRAFFITI|        NOGRAFFITI2|SINGLEPLAYER
    .byte NOGRAFFITI|SPEEDUP|NOGRAFFITI2
    .byte NOGRAFFITI|SPEEDUP|NOGRAFFITI2|SINGLEPLAYER
    .byte NOGRAFFITI|        NOGRAFFITI2|                              DIAGONAL
    .byte NOGRAFFITI|SPEEDUP|NOGRAFFITI2|                              DIAGONAL
    .byte NOGRAFFITI|SPEEDUP|NOGRAFFITI2|                        ERASE|DIAGONAL
    .byte NOGRAFFITI|        NOGRAFFITI2|             WRAPAROUND
    .byte NOGRAFFITI|SPEEDUP|NOGRAFFITI2|             WRAPAROUND
    .byte NOGRAFFITI|        NOGRAFFITI2|             WRAPAROUND|      DIAGONAL
    .byte NOGRAFFITI|SPEEDUP|NOGRAFFITI2|             WRAPAROUND|      DIAGONAL
    .byte NOGRAFFITI|SPEEDUP|NOGRAFFITI2|             WRAPAROUND|ERASE|DIAGONAL
    .byte                                             WRAPAROUND|ERASE
    .byte                                             WRAPAROUND|ERASE|DIAGONAL

; some free bytes:
    .ds   77,0

    ORG $f7fa

Vectors:
    .byte $94,$f3,$94,$f3,$94,$f3
